/*
 * arch/arm/mach-mb86hxx/include/mach/memory.h
 *
 * Copyright (C) 2008-2010 FUJITSU MICROELECTRONICS LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef __ARCH_ARM_MACH_MEMORY_H
#define __ARCH_ARM_MACH_MEMORY_H

#include <mach/hardware.h>

/*
 * Physical DRAM offset, size.
 */
#ifdef CONFIG_MB86HXX_LARGEMEM
#define PHYS_OFFSET		(0x20000000 + CONFIG_FIRM_AREA_SIZE)
#define PHYS_SIZE		CONFIG_MEM_SIZE
#define MEM_SIZE		(CONFIG_MEM_SIZE - CONFIG_FIRM_AREA_SIZE)
#else
#define PHYS_OFFSET		UL(0x20000000)
#define PHYS_SIZE		UL(0x4000000)
#define MEM_SIZE		CONFIG_MEM_SIZE
#endif /* CONFIG_MB86HXX_LARGEMEM */

#ifdef CONFIG_MB86H6X_MEMHOLE
/*
 * discontinuous memory layout
 * PHYS                                                 VIRT
 * ====                                                 ====
 * PHYS_OFFSET-(PHYS_OFFSET+MEM_SIZE)                   PAGE_OFFSET
 * (PHYS_OFFSET+MEM_SIZE)-(PHYS_OFFSET+PHYS_SIZE)       not mapped (i.e. hole)
 * (PHYS_OFFSET+PHYS_SIZE)-0x28000000                   (PAGE_OFFSET+MEM_SIZE)
 */
#define __phys_to_virt(x)   ((x) >= (PHYS_OFFSET+PHYS_SIZE) ?		   \
		((x) - (PHYS_OFFSET+PHYS_SIZE) + (PAGE_OFFSET+MEM_SIZE)) : \
		((x) - PHYS_OFFSET + PAGE_OFFSET))
#define __virt_to_phys(x)   ((x) >= (PAGE_OFFSET+MEM_SIZE) ?		   \
		((x) - (PAGE_OFFSET+MEM_SIZE) + (PHYS_OFFSET+PHYS_SIZE)) : \
		((x) - PAGE_OFFSET + PHYS_OFFSET))

#define pfn_valid(pfn)       ( ( ((pfn) >= __phys_to_pfn(PHYS_OFFSET)) &&   \
                  ((pfn) <  __phys_to_pfn(PHYS_OFFSET+MEM_SIZE))    )    || \
                 (((pfn) >= __phys_to_pfn(PHYS_OFFSET+PHYS_SIZE))  &&       \
                  ((pfn) <  __phys_to_pfn(PHYS_OFFSET+PHYS_SIZE*2)) ) )
#endif /* CONFIG_MB86H6X_MEMHOLE */

/*
 * Virtual view <-> DMA view memory address translations
 * virt_to_bus: Used to translate the virtual address to an
 *              address suitable to be passed to set_dma_addr
 * bus_to_virt: Used to convert an address for DMA operations
 *              to an address that the kernel can use.
 */
#define __virt_to_bus(x)	__virt_to_phys(x)
#define __bus_to_virt(x)	__phys_to_virt(x)

#define IO_PHYS_TO_VIRT(x)      io_phys_to_virt(x)
#define IO_VIRT_TO_PHYS(x)      io_virt_to_phys(x)

#ifndef __ASSEMBLY__
extern unsigned long io_phys_to_virt(unsigned long x);
extern unsigned long io_virt_to_phys(unsigned long x);
#endif /* __ASSEMBLY__ */

#endif				/* __ARCH_ARM_MACH_MEMORY_H */
